;==========================================================================

    include \masm32\include\masm32rt.inc
    include winio.inc
    includelib winio.lib
    include smbios.inc

    ; This GPFs under WindowsXP, so it's basically a waste of time.
    ; A RM DOS app should work if the structure table is in the RM
    ; address space, othewise a PM app will be required.

;==========================================================================

stype MACRO saddr
    mov eax, saddr
    movzx eax, [eax].SMB_STRUCTURE_HEADER.structureType
    exitm <eax>
ENDM

debugprint MACRO quotedtext, dwordval
    pushad
    IFB <dwordval>
        print quotedtext,13,10
    ELSE
        print quotedtext,9
        mov ebx, dwordval
        print ustr$(ebx),13,10
    ENDIF
    invoke Sleep, 200
    popad
ENDM

;==========================================================================

    .data
      hPhysMem              dd 0
      structureTableLength  dd 0
      structureTableAddress dd 0
      numberOfStructures    dd 0
      bcdRevision           dd 0
      hInst                 dd 0
      buff                  db 100 dup(0)
    .code

;==========================================================================
;
; NEED TO USE NUMBER OF STRUCTURES TO RECOGNIZE END OF TABLE
; IGNORE SMB_TYPE_END_OF_TABLE AND SMB_TYPE_INACTIVE FOR NOW
;
; ACCESS BY TABLE ADDRESS, WITH STRUCTURES AS TEMPLATE
; SO OBVIOUSLY NEED TO RETURN ADDRESS, OR ZERO IF NOT FOUND
; AND CALL IT TABLEADDRESS INSTEAD OF START ADDRESS
;
; IF SAVE RETURN ADDRESS AND STRUCTURETYPE TO GLOBAL,
; THEN CAN USE FINDNEXTSTRUCTURE WITHOUT PARAMETERS
;
; NEED TO ALSO SAVE STRUCTURE NUMBER TO GLOBAL SO
;  FINDNEXTSTRUCTURE CAN KNOW WHEN TO STOP
; AND FINDNEXTSTRUCTURE NEEDS TO UPDATE GLOBAL STRUCTURE ADDRESS
;
; SO NUTIN' TO IT

FindStructure proc tableAddress:DWORD,
                   structureCount:DWORD,
                   structureType:DWORD

    mov eax, tableAddress
    mov ecx, structureType
  @@:
    cmp [eax].SMB_STRUCTURE_HEADER.structureType, cl
    je  @F

    ;---------------------------------------------------
    ; Skip to the start of the text strings that follow
    ; the formatted portion of the structure.
    ;---------------------------------------------------

    movzx edx, [eax].SMB_STRUCTURE_HEADER.formattedLength
    add eax, edx

    ;------------------------------------------------------------
    ; Scan to the double null that marks the end of the strings.
    ;------------------------------------------------------------

    .WHILE WORD PTR [eax]
        inc eax
    .ENDW

    ;----------------------------
    ; Skip over the double null.
    ;----------------------------

    add eax, 2

    ;-------------------------------------------
    ; Repeat until have scanned last structure.
    ;-------------------------------------------

    dec structureCount
    jnz @B

    xor eax, eax
  @@:
    ret

FindStructure endp

;==========================================================================
; This can be made to work without the globals
; Need to pass structure count
; Then again, could also just add a flag

comment |
FindNextStructure proc tableAddress:DWORD, structureType:DWORD

    mov eax, tableAddress
    movzx edx, [eax].SMB_STRUCTURE_HEADER.formattedLength
    add eax, edx
    .WHILE WORD PTR [eax]
      inc eax
    .ENDW
    add eax, 2
    .IF stype(eax) == SMB_TYPE_END_OF_TABLE
      xor eax, eax
    .ELSE
      invoke FindStructure, tableAddress, structureType
    .ENDIF

    ret

FindNextStructure endp
|

;==========================================================================

FindString proc structureAddress:DWORD, stringIndex:DWORD

    mov eax, structureAddress
    movzx edx, [eax].SMB_STRUCTURE_HEADER.formattedLength
    add eax, edx
    mov ecx, stringIndex
    .WHILE ecx > 1
        .WHILE BYTE PTR [eax] != 0
            inc eax
        .ENDW
        inc eax
        dec ecx
    .ENDW
  @@:
    ret

FindString endp

;==========================================================================
start:
;==========================================================================

    invoke InitializeWinIo

    invoke MapPhysToLin, 0F0000h, 10000h, ADDR hPhysMem
    mov   ebx, eax

    xor esi, esi
    .WHILE esi < 10000h

      .IF DWORD PTR [ebx+esi] == "_MS_"

        print "SMBIOS version: "
        xor eax, eax
        mov al,[ebx+esi].SMB_ENTRY_POINT_STRUCTURE.smbMajorVersion
        print ustr$(eax),"."
        mov al,[ebx+esi].SMB_ENTRY_POINT_STRUCTURE.smbMinorVersion
        print ustr$(eax),13,10

        movzx eax,[ebx+esi].SMB_ENTRY_POINT_STRUCTURE.structureTableLength
        mov structureTableLength, eax
        mov eax,[ebx+esi].SMB_ENTRY_POINT_STRUCTURE.structureTableAddress
        mov structureTableAddress, eax
        movzx eax,[ebx+esi].SMB_ENTRY_POINT_STRUCTURE.numberOfStructures
        mov numberOfStructures, eax

        print "structure table length: "
        print ustr$(structureTableLength),13,10
        print "structure table address: "
        print uhex$(structureTableAddress),13,10
        print "structure table number of structures: "
        print ustr$(numberOfStructures),13,10,13,10

        print "BCD revision: "
        movzx eax,[ebx+esi].SMB_ENTRY_POINT_STRUCTURE.bcdRevision
        mov bcdRevision, eax
        print uhex$(eax),13,10,13,10

        .BREAK

      .ENDIF

      add esi, 16

    .ENDW

    .IF esi >= 10000h

      print "SMBIOS Structure Table Entry Point not found",13,10
      inkey "Press any key to exit..."
      exit

    .ENDIF

    ;---------------------------------------------------
    ; The structure table can be at any 32-bit address.
    ;---------------------------------------------------


    ; F1F30h


    invoke UnmapPhysicalMemory, hPhysMem, ebx

    ;---------------------------------------------------------------------
    ; For some reason, mapping the amount of physical memory specified in
    ; the structureTableLength member of the entry point structure will
    ; result in an access violation. Even though a somewhat smaller value
    ; worked in my tests, I added the value 4096 based on a guess that the
    ; problem has something to do with page alignment.
    ;---------------------------------------------------------------------

    mov eax, structureTableLength
    add eax, 4096
    invoke MapPhysToLin, structureTableAddress,
                         eax,
                         ADDR hPhysMem
    mov ebx, eax
    print "Linear address: "
    print uhex$(ebx),13,10,13,10

    invoke FindStructure, ebx, numberOfStructures, SMB_TYPE_BIOS_INFO
    .IF eax

        mov esi, eax

        print "SMB_BIOS_INFO structure address: "
        print uhex$(esi),13,10

        print "vendor: "
        movzx eax, [esi].SMB_BIOS_INFO.biVendor$
        invoke FindString, esi, eax
        print eax,13,10

        print "version: "
        movzx eax, [esi].SMB_BIOS_INFO.biVersion$
        invoke FindString, esi, eax
        print eax,13,10

        print "release date: "
        movzx eax, [esi].SMB_BIOS_INFO.biReleaseDate$
        invoke FindString, esi, eax
        print eax,13,10,13,10

    .ENDIF

    invoke FindStructure, ebx, numberOfStructures, SMB_TYPE_SYSTEM_INFO
    .IF eax

        mov esi, eax
        print "SMB_TYPE_SYSTEM_INFO serial number "
        invoke FindString, esi, 4       ; serialNumber$
        print eax,13,10,13,10
    .ENDIF

    invoke FindStructure, ebx, numberOfStructures, SMB_TYPE_PROCESSOR_INFO
    .IF eax

        mov esi, eax
        print "SMB_PROCESSOR_INFO structure address: "
        print uhex$(esi),13,10

        print "socket: "
        movzx eax, [esi].SMB_PROCESSOR_INFO.piSocket$
        invoke FindString, esi, eax
        print eax,13,10

        invoke GetModuleHandle, NULL
        mov hInst, eax

        print "type: "
        movzx eax, [esi].SMB_PROCESSOR_INFO.piType
        SWITCH eax
          CASE PITYPE_OTHER
            print "PITYPE_OTHER",13,10
          CASE PITYPE_UNKNOWN
            print "PITYPE_UNKNOWN",13,10
          CASE PITYPE_CENTRAL
            print "PITYPE_CENTRAL",13,10
          CASE PITYPE_MATH
            print "PITYPE_MATH",13,10
          CASE PITYPE_DSP
            print "PITYPE_DSP",13,10
          CASE PITYPE_VIDEO
            print "PITYPE_VIDEO",13,10
        ENDSW

;; THIS NEEDS TO CONSIDER FAMILY2

        print "family: "
        movzx eax, [esi].SMB_PROCESSOR_INFO.piFamily
        invoke LoadString, hInst, eax, ADDR buff, SIZEOF buff
        print ADDR buff,13,10

        print "manufacturer: "
        movzx eax, [esi].SMB_PROCESSOR_INFO.piManufacturer$
        invoke FindString, esi, eax
        print eax,13,10

        .IF bcdRevision >= 25h
            print "core count: "
            movzx eax, [esi].SMB_PROCESSOR_INFO.piCoreCount
            print ustr$(eax),13,10
        .ENDIF

    .ENDIF

    invoke UnmapPhysicalMemory, hPhysMem, ebx

    invoke ShutdownWinIo

    print chr$(13,10)

    inkey "Press any key to exit..."
    exit

;==========================================================================
end start